@@ -23,6 +23,7 @@
 #define _MPEGACTION_H_
 
 #include "SDL.h"
+#include "MPEGfilter.h"
 
 typedef enum {
     MPEG_ERROR = -1,
@@ -36,6 +37,7 @@ public:
         playing = false;
         paused = false;
         looping = false;
+	play_time = 0.0;
     }
     virtual void Loop(bool toggle) {
         looping = toggle;
@@ -46,6 +48,8 @@ public:
     virtual void Play(void) = 0;
     virtual void Stop(void) = 0;
     virtual void Rewind(void) = 0;
+    virtual void ResetSynchro(double) = 0;
+    virtual void Skip(float seconds) = 0;
     virtual void Pause(void) {  /* A toggle action */
         if ( paused ) {
             paused = false;
@@ -70,7 +74,12 @@ protected:
 
 /* For getting info about the audio portion of the stream */
 typedef struct MPEG_AudioInfo {
-    int unused;
+    int mpegversion;
+    int mode;
+    int frequency;
+    int layer;
+    int bitrate;
+    int current_frame;
 } MPEG_AudioInfo;
 
 /* Audio action class */
@@ -106,11 +115,22 @@ public:
     virtual bool SetDisplay(SDL_Surface *dst, SDL_mutex *lock,
                                 MPEG_DisplayCallback callback) = 0;
     virtual void MoveDisplay(int x, int y) = 0;
-    virtual void DoubleDisplay(bool toggle) = 0;
-    virtual void RenderFrame(int frame, SDL_Surface *dst, int x, int y) = 0;
+    virtual void ScaleDisplayXY(int w, int h) = 0;
+    virtual void SetDisplayRegion(int x, int y, int w, int h) = 0;
+    virtual void RenderFrame(int frame) = 0;
     virtual void RenderFinal(SDL_Surface *dst, int x, int y) = 0;
+    virtual SMPEG_Filter * Filter(SMPEG_Filter * filter) = 0;
 protected:
     MPEGaudioaction *time_source;
 };
 
+
+/* For getting info about the system portion of the stream */
+typedef struct MPEG_SystemInfo {
+    int total_size;
+    int current_offset;
+    double total_time;
+    double current_time;
+} MPEG_SystemInfo;
+
 #endif /* _MPEGACTION_H_ */
