@@ -22,17 +22,16 @@
 #ifndef _MPEGAUDIO_H_
 #define _MPEGAUDIO_H_
 
-/* This improves the performance of the audio player by quite a bit */
-#define THREADED_AUDIO
-
 #include "SDL.h"
-#include "MPEGstream.h"
+#include "MPEGerror.h"
 #include "MPEGaction.h"
 
 #ifdef THREADED_AUDIO
 #include "MPEGring.h"
 #endif
 
+class MPEGstream;
+
 /* MPEG/WAVE Sound library
 
    (C) 1997 by Woo-jae Jung */
@@ -134,11 +133,7 @@ public:
       register unsigned short a;
       { int offset=bitindex>>3;
 
-#if SDL_BYTEORDER == SDL_LIL_ENDIAN
         a=(((unsigned char)buffer[offset])<<8) | ((unsigned char)buffer[offset+1]);
-#else
-        a=((unsigned short *)buffer)[offset/2];
-#endif
       }
       a<<=(bitindex&7);
       bitindex+=bits;
@@ -154,7 +149,8 @@ private:
 /* The actual MPEG audio class */
 class MPEGaudio : public MPEGerror, public MPEGaudioaction {
 
-    friend void Play_MPEGaudio(void *udata, Uint8 *stream, int len);
+    friend void Play_MPEGaudioSDL(void *udata, Uint8 *stream, int len);
+    friend int Play_MPEGaudio(MPEGaudio *audio, Uint8 *stream, int len);
 #ifdef THREADED_AUDIO
     friend int Decode_MPEGaudio(void *udata);
 #endif
@@ -164,10 +160,13 @@ public:
     virtual ~MPEGaudio();
 
     /* MPEG actions */
+    bool GetAudioInfo(MPEG_AudioInfo *info);
     double Time(void);
     void Play(void);
     void Stop(void);
     void Rewind(void);
+    void ResetSynchro(double time);
+    void Skip(float seconds);
     void Volume(int vol);
     MPEGstatus Status(void);
 
@@ -182,11 +181,16 @@ protected:
     bool sdl_audio;
     MPEGstream *mpeg;
     int valid_stream;
+    bool stereo;
     double rate_in_s;
     Uint32 frags_playing;
     Uint32 frag_time;
 #ifdef THREADED_AUDIO
+    bool decoding;
     SDL_Thread *decode_thread;
+
+    void StartDecoding(void);
+    void StopDecoding(void);
 #endif
 
 /* Code from splay 1.8.2 */
@@ -242,7 +246,7 @@ private:
   /*******************/
 public:
   void initialize();
-  bool run(int frames);
+  bool run(int frames, double *timestamp = NULL);
   void clearbuffer(void);
 
   /*****************************/
@@ -250,54 +254,15 @@ public:
   /*****************************/
 private:
   Uint8 _buffer[4096];
+  Uint32 _buffer_pos;
   int  bitindex;
-  bool fillbuffer(int size)
-  {
-      bitindex=0;
-      return(mpeg->copy_data(_buffer, size) > 0);
-  };
-  void sync(void)  {bitindex=(bitindex+7)&0xFFFFFFF8;};
-  bool issync(void){return (bitindex&7);};
-  int getbyte(void) {
-      int r=(unsigned char)_buffer[bitindex>>3];
-
-      bitindex+=8;
-      return r;
-  }
-  int getbit(void) {
-      register int r=(_buffer[bitindex>>3]>>(7-(bitindex&7)))&1;
-
-      bitindex++;
-      return r;
-  }
-  int getbits8(void) {
-      register unsigned short a;
-      { int offset=bitindex>>3;
-
-#if SDL_BYTEORDER == SDL_LIL_ENDIAN
-        a=(((unsigned char)_buffer[offset])<<8) | ((unsigned char)_buffer[offset+1]);
-#else
-        a=((unsigned short *)_buffer)[offset/2];
-#endif
-      }
-      a<<=(bitindex&7);
-      bitindex+=8;
-      return (int)((unsigned int)(a>>8));
-  }
-  int getbits9(int bits) {
-      register unsigned short a;
-      { int offset=bitindex>>3;
-
-#if SDL_BYTEORDER == SDL_LIL_ENDIAN
-        a=(((unsigned char)_buffer[offset])<<8) | ((unsigned char)_buffer[offset+1]);
-#else
-        a=((unsigned short *)_buffer)[offset/2];
-#endif
-      }
-      a<<=(bitindex&7);
-      bitindex+=bits;
-      return (int)((unsigned int)(a>>(16-bits)));
-  }
+  bool fillbuffer(int size);
+  void sync(void);
+  bool issync(void);
+  int getbyte(void);
+  int getbit(void);
+  int getbits8(void);
+  int getbits9(int bits);
   int getbits(int bits);
 
 
@@ -377,9 +342,10 @@ private:
   int     samplesperframe;
   int     rawdatareadoffset, rawdatawriteoffset;
   Sint16 *rawdata;
-  Sint16  spillover[ RAWDATASIZE ];
 #ifdef THREADED_AUDIO
   MPEG_ring *ring;
+#else
+  Sint16  spillover[ RAWDATASIZE ];
 #endif
   int volume;
 
@@ -389,6 +355,14 @@ private:
         rawdata=NULL;
   }
   void putraw(short int pcm) {rawdata[rawdatawriteoffset++]=pcm;}
+
+  /********************/
+  /* Timestamp sync   */
+  /********************/
+public:
+#define N_TIMESTAMPS 5
+
+  double timestamp[N_TIMESTAMPS];
 };
 
 #endif /* _MPEGAUDIO_H_ */
