@@ -24,23 +24,26 @@
 
 #include "SDL.h"
 #include "SDL_thread.h"
-#include "MPEGstream.h"
+#include "MPEGerror.h"
 #include "MPEGaction.h"
 
+class MPEGstream;
 
 /* This is the MPEG video stream structure in the mpeg_play code */
 struct vid_stream;
 typedef struct vid_stream VidStream;
 
+/* Temporary definition of time stamp structure. */
+
+typedef double TimeStamp;
+
 class MPEGvideo : public MPEGerror, public MPEGvideoaction {
 
     /* Thread to play the video asynchronously */
     friend int Play_MPEGvideo(void *udata);
 
     /* Various mpeg_play functions that need our data */
-    friend void DoDitherImage( VidStream* vid_stream );
-    friend void DisplayCurrentFrame( VidStream* vid_stream );
-    friend int timeSync( VidStream* vid_stream );
+    friend VidStream* mpegVidRsrc( TimeStamp time_stamp, VidStream* vid_stream, int first );
     friend int get_more_data( VidStream* vid_stream );
 
 public:
@@ -48,10 +51,11 @@ public:
     virtual ~MPEGvideo();
 
     /* MPEG actions */
-    void Loop(bool toggle);
     void Play(void);
     void Stop(void);
     void Rewind(void);
+    void ResetSynchro(double time);
+     void Skip(float seconds);
     MPEGstatus Status(void);
 
     /* MPEG video actions */
@@ -59,9 +63,16 @@ public:
     bool SetDisplay(SDL_Surface *dst, SDL_mutex *lock,
                                             MPEG_DisplayCallback callback);
     void MoveDisplay(int x, int y);
-    void DoubleDisplay(bool toggle);
-    void RenderFrame(int frame, SDL_Surface *dst, int x, int y);
+    void ScaleDisplayXY(int w, int h);
+    void SetDisplayRegion(int x, int y, int w, int h);
+    void RenderFrame(int frame);
     void RenderFinal(SDL_Surface *dst, int x, int y);
+    SMPEG_Filter * Filter(SMPEG_Filter * filter);
+
+    /* Display and sync functions */
+    void DisplayFrame( VidStream* vid_stream );
+    void ExecuteDisplay( VidStream* vid_stream );
+    int timeSync( VidStream* vid_stream );
 
     /* Yes, it's a hack.. */
     MPEGaudioaction *TimeSource(void ) {
@@ -72,23 +83,22 @@ protected:
     MPEGstream *mpeg;
 
     VidStream* _stream;
-    SDL_Surface* _surf;
+    SDL_Surface* _dst;
     SDL_mutex* _mutex;
     SDL_Thread* _thread;
 
     MPEG_DisplayCallback _callback;
 
-    int _double;        // play back at double size
-    int _w;             // width of movie
-    int _h;             // height of movie
-    int _x;             // pixel x offset
-    int _y;             // pixel y offset
-    int _uw;            // update width
-    int _uh;            // update height
-
-    int _lum[ 8 ];  	// hardcoded LUM_RANGE = 8
-    int _cr[ 4 ];   	// hardcoded CR_RANGE = 4
-    int _cb[ 4 ];   	// hardcoded CB_RANGE = 4
+    int _ow;            // original width of the movie
+    int _oh;            // original height of the movie
+    int _w;             // mb aligned width of the movie
+    int _h;             // mb aligned height of the movie
+    SDL_Rect _srcrect;	// source area
+    SDL_Rect _dstrect;	// display area
+    SDL_Overlay *_image;// source image
+    float _fps;         // frames per second
+    SMPEG_Filter * _filter; // pointer to the current filter used
+    SDL_mutex* _filter_mutex; // make sure the filter is not changed while being used
 
     void RewindStream(void);
 };
