@@ -20,11 +20,7 @@
 /* A class based on the MPEG stream class, used to parse and play audio */
 
 #include "MPEGaudio.h"
-
-#ifdef SDL_MIXER             /* From the SDL mixer example library */
-#include "SDL_mixer.h"
-#endif
-
+#include "MPEGstream.h"
 
 MPEGaudio:: MPEGaudio(MPEGstream *stream, bool initSDL) : sdl_audio(initSDL)
 {
@@ -37,8 +33,6 @@ MPEGaudio:: MPEGaudio(MPEGstream *stream, bool initSDL) : sdl_audio(initSDL)
 
     /* Analyze the MPEG audio stream */
     if ( loadheader() ) {
-        mpeg->reset_stream();
-
         SDL_AudioSpec wanted;
         WantedSpec(&wanted);
 
@@ -59,41 +53,37 @@ MPEGaudio:: MPEGaudio(MPEGstream *stream, bool initSDL) : sdl_audio(initSDL)
             /* Open the audio, get actual audio hardware format and convert */
             bool audio_active;
             SDL_AudioSpec actual;
-#ifdef SDL_MIXER
-            Mix_OpenAudio(wanted.freq,wanted.format,wanted.channels,wanted.samples);
-            int actual_channels;    /* This is stupid, need to fix the API */
-            audio_active = Mix_QuerySpec(&actual.freq,
-                                         &actual.format, &actual_channels);
-            actual.channels = actual_channels;
-#else
             audio_active = (SDL_OpenAudio(&wanted, &actual) == 0);
-            SDL_PauseAudio(0);
-#endif
             if ( audio_active ) {
                 ActualSpec(&actual);
                 valid_stream = true;
             } else {
                 SetError(SDL_GetError());
             }
-        } else { /* The stream is always valid if we don't initiliaze SDL */
+            SDL_PauseAudio(0);
+        } else { /* The stream is always valid if we don't initialize SDL */
             valid_stream = true; 
         }
         Volume(100);
     }
+
+    /* For using system timestamp */
+    for (int i=0; i<N_TIMESTAMPS; i++)
+      timestamp[i] = -1;
 }
 
 MPEGaudio:: ~MPEGaudio()
 {
+#ifdef THREADED_AUDIO
+    /* Stop the decode thread */
+    StopDecoding();
+#endif
+
     /* Remove ourselves from the mixer hooks */
     Stop();
-
     if ( sdl_audio ) {
         /* Close up the audio so others may play */
-#ifdef SDL_MIXER
-        Mix_CloseAudio();
-#else
         SDL_CloseAudio();
-#endif
     }
 }
 
@@ -112,7 +102,7 @@ MPEGaudio:: WantedSpec(SDL_AudioSpec *wanted)
         wanted->channels = 1;
     }
     wanted->samples = 4096;
-    wanted->callback = Play_MPEGaudio;
+    wanted->callback = Play_MPEGaudioSDL;
     wanted->userdata = this;
     return true;
 }
@@ -132,7 +122,12 @@ MPEGaudio:: ActualSpec(const SDL_AudioSpec *actual)
     if ( (actual->freq/100) == ((frequencies[version][frequency]/2)/100) ) {
         downfrequency = 1;
     } else if ( actual->freq != frequencies[version][frequency] ) {
-        fprintf(stderr, "Warning: incorrect audio frequency\n");
+#ifdef VERBOSE_WARNINGS
+        fprintf(stderr, "Warning: wrong audio frequency (wanted %d, got %d)\n",
+		frequencies[version][frequency], actual->freq);
+#else
+	;
+#endif
     }
 #if SDL_BYTEORDER == SDL_LIL_ENDIAN
     if ( actual->format != AUDIO_S16LSB)
@@ -142,9 +137,39 @@ MPEGaudio:: ActualSpec(const SDL_AudioSpec *actual)
     {
         fprintf(stderr, "Warning: incorrect audio format\n");
     }
-    rate_in_s = (double)(((actual->format&0xFF)/8)*actual->channels*actual->freq);
+    rate_in_s=((double)((actual->format&0xFF)/8)*actual->channels*actual->freq);
+    stereo=((actual->channels-1) > 0);
 }
 
+#ifdef THREADED_AUDIO
+void
+MPEGaudio:: StartDecoding(void)
+{
+    decoding = true;
+    /* Create the ring buffer to hold audio */
+    if ( ! ring ) {
+        ring = new MPEG_ring(samplesperframe*2);
+    }
+    if ( ! decode_thread ) {
+        decode_thread = SDL_CreateThread(Decode_MPEGaudio, this);
+    }
+}
+void
+MPEGaudio:: StopDecoding(void)
+{
+    decoding = false;
+    if ( decode_thread ) {
+        if( ring ) ring->ReleaseThreads();
+        SDL_WaitThread(decode_thread, NULL);
+        decode_thread = NULL;
+    }
+    if ( ring ) {
+        delete ring;
+        ring = NULL;
+    }
+}
+#endif
+
 /* MPEG actions */
 double
 MPEGaudio:: Time(void)
@@ -154,7 +179,7 @@ MPEGaudio:: Time(void)
     if ( frag_time ) {
         now = (play_time + (double)(SDL_GetTicks() - frag_time)/1000.0);
     } else {
-        now = 0.0;
+        now = play_time;
     }
     return now;
 }
@@ -163,56 +188,19 @@ MPEGaudio:: Play(void)
 {
     ResetPause();
     if ( valid_stream ) {
-        if (Status() == MPEG_PLAYING) {
-            Stop();
-        }
 #ifdef THREADED_AUDIO
-        /* Create the ring buffer to hold audio */
-        ring = new MPEG_ring(samplesperframe*2);
+        StartDecoding();
 #endif
         playing = true;
-
-#ifdef THREADED_AUDIO
-        /* Start the decoding thread */
-        decode_thread = SDL_CreateThread(Decode_MPEGaudio, this);
-#endif
-#ifdef SDL_MIXER
-        if ( sdl_audio ) {
-            /* Hook ourselves up to the mixer */
-            Mix_HookMusic(Play_MPEGaudio, this);
-        }
-#endif
     }
 }
 void
 MPEGaudio:: Stop(void)
 {
     if ( valid_stream ) {
+        SDL_LockAudio();
         playing = false;
-#ifdef THREADED_AUDIO
-        /* Release the threads */
-        if ( ring ) {
-            ring->ReleaseThreads();
-        }
-
-        /* Stop the decoding thread */
-        if ( decode_thread ) {
-            SDL_WaitThread(decode_thread, NULL);
-            decode_thread = NULL;
-        }
-#endif
-#ifdef SDL_MIXER
-        /* Unhook ourselves from the mixer */
-        if ( sdl_audio && Mix_GetMusicHookData() == this ) {
-            Mix_HookMusic(NULL, NULL);
-        }
-#endif
-#ifdef THREADED_AUDIO
-        if ( ring ) {
-            delete ring;
-            ring = NULL;
-        }
-#endif
+        SDL_UnlockAudio();
     }
     ResetPause();
 }
@@ -220,15 +208,39 @@ void
 MPEGaudio:: Rewind(void)
 {
     Stop();
-    mpeg->reset_stream();
+
+#ifdef THREADED_AUDIO
+    /* Stop the decode thread */
+    StopDecoding();
+#endif
+
     clearrawdata();
     decodedframe = 0;
     currentframe = 0;
     frags_playing = 0;
-    frag_time = 0;
-    play_time = 0.0;
 }
 void
+MPEGaudio:: ResetSynchro(double time)
+{
+    play_time = time;
+    frag_time = 0;
+
+    /* Reinit the timestamp FIFO */
+    for (int i=0; i<N_TIMESTAMPS; i++)
+      timestamp[i] = -1;
+}
+void
+MPEGaudio:: Skip(float seconds)
+{
+   /* Called only when there is no timestamp info in the MPEG */
+   printf("Audio: Skipping %f seconds...\n", seconds);
+   while(seconds > 0)
+   {
+     seconds -= (float) samplesperframe / ((float) frequencies[version][frequency]*(1+inputstereo));
+     if(!loadheader()) break;
+   }
+ }
+void
 MPEGaudio:: Volume(int vol)
 {
     if ( (vol >= 0) && (vol <= 100) ) {
@@ -253,3 +265,80 @@ MPEGaudio:: Status(void)
         return(MPEG_ERROR);
     }
 }
+
+bool
+MPEGaudio:: GetAudioInfo(MPEG_AudioInfo *info)
+{
+    if ( info ) {
+      info->mpegversion = version;
+      info->mode = mode;
+      info->frequency = frequencies[version][frequency];
+      info->layer = layer;
+      info->bitrate = bitrate[version][layer-1][bitrateindex];
+      info->current_frame = currentframe;
+    }
+    return true;
+}
+bool
+MPEGaudio:: fillbuffer(int size)
+  {
+      bitindex=0;
+      _buffer_pos = mpeg->pos;
+      return(mpeg->copy_data(_buffer, size) > 0);
+  };
+  
+void
+MPEGaudio:: sync(void)
+{
+  bitindex=(bitindex+7)&0xFFFFFFF8;
+}
+  
+bool
+MPEGaudio:: issync(void)
+{
+  return (bitindex&7) != 0;
+}
+  
+int 
+MPEGaudio::getbyte(void) 
+{
+  int r=(unsigned char)_buffer[bitindex>>3];
+
+  bitindex+=8;
+  return r;
+}
+  
+int 
+MPEGaudio::getbit(void) 
+{
+  register int r=(_buffer[bitindex>>3]>>(7-(bitindex&7)))&1;
+
+  bitindex++;
+  return r;
+}
+  
+int 
+MPEGaudio::getbits8(void) 
+{
+  register unsigned short a;
+  { int offset=bitindex>>3;
+
+  a=(((unsigned char)_buffer[offset])<<8) | ((unsigned char)_buffer[offset+1]);
+  }
+  a<<=(bitindex&7);
+  bitindex+=8;
+  return (int)((unsigned int)(a>>8));
+}
+  
+int 
+MPEGaudio::getbits9(int bits) 
+{
+  register unsigned short a;
+  { int offset=bitindex>>3;
+
+  a=(((unsigned char)_buffer[offset])<<8) | ((unsigned char)_buffer[offset+1]);
+  }
+  a<<=(bitindex&7);
+  bitindex+=bits;
+  return (int)((unsigned int)(a>>(16-bits)));
+}
