@@ -78,7 +78,6 @@
 */
 
 
-
 #include <limits.h>
 #include <string.h>
 
@@ -88,7 +87,7 @@
 #include "util.h"
 
 #include "MPEGvideo.h"
-
+#include "MPEGfilter.h"
 
 /*--------------------------------------------------------------*/
 
@@ -106,15 +105,11 @@ int quietFlag = 1;
                0: as fast as possible
                N (N>0): N frames/sec  
                */
+#ifdef TIME_MPEG
+int framerate = 0;
+#else
 int framerate = -1;
-
-/* Flag for gamma correction */
-int gammaCorrectFlag = 0;
-double gammaCorrect = 1.0;
-
-/* Flag for chroma correction */
-int chromaCorrectFlag = 0;
-double chromaCorrect = 1.0;
+#endif
 
 /* Flag for high quality at the expense of speed */
 #ifdef QUALITY
@@ -128,6 +123,7 @@ int qualityFlag = 0;
 MPEGvideo::MPEGvideo(MPEGstream *stream)
 {
     Uint32 start_code;
+    MPEGstream_marker *marker;
 
     /* Set the MPEG data stream */
     mpeg = stream;
@@ -135,13 +131,14 @@ MPEGvideo::MPEGvideo(MPEGstream *stream)
 
     /* Set default playback variables */
     _thread = NULL;
-    _x = 0;
-    _y = 0;
-    _double = false;
-    _surf = NULL;
+    _dst = NULL;
     _mutex = NULL;
     _stream = NULL;
 
+    /* Mark the data to leave the stream unchanged */
+    /* after parsing */
+    marker = mpeg->new_marker(0);
+
     /* Get the width and height of the video */
     start_code = mpeg->copy_byte();
     start_code <<= 8;
@@ -157,14 +154,56 @@ MPEGvideo::MPEGvideo(MPEGstream *stream)
 
         /* Get the width and height of the video */
         mpeg->copy_data(buf, 4);
-        _w = (buf[0]<<4)|(buf[1]>>4);     /* 12 bits of width */
-        _h = ((buf[1]&0xF)<<8)|buf[2];    /* 12 bits of height */
+        _w = (buf[0]<<4)|(buf[1]>>4);    /* 12 bits of width */
+        _h = ((buf[1]&0xF)<<8)|buf[2];   /* 12 bits of height */
+	switch(buf[3]&0xF)                /*  4 bits of fps */
+	{
+	  case 1: _fps = 23.97f; break;
+	  case 2: _fps = 24.00f; break;
+	  case 3: _fps = 25.00f; break;
+	  case 4: _fps = 29.97f; break;
+	  case 5: _fps = 30.00f; break;
+	  case 6: _fps = 50.00f; break;
+	  case 7: _fps = 59.94f; break;
+	  case 8: _fps = 60.00f; break;
+	  case 9: _fps = 15.00f; break;
+	  default: _fps = 30.00f; break;
+	}
     } else {
         _w = 0;
         _h = 0;
+	_fps = 0.00;
         SetError("Not a valid MPEG video stream");
     }
-    mpeg->reset_stream();
+    /* Rewind back to the old position */
+    mpeg->seek_marker(marker);
+    mpeg->delete_marker(marker);
+
+    /* Keep original width and height in _ow and _oh */
+    _ow = _w;
+    _oh = _h;
+
+    /* Now round up width and height to a multiple   */
+    /* of a macroblock size (16 pixels) to keep the  */
+    /* video decoder happy */
+    _w = (_w + 15) & ~15;
+    _h = (_h + 15) & ~15;
+
+    /* Set the default playback area */
+    _dstrect.x = 0;
+    _dstrect.y = 0;
+    _dstrect.w = 0;
+    _dstrect.h = 0;
+
+    /* Set the source area */
+    _srcrect.x = 0;
+    _srcrect.y = 0;
+    _srcrect.w = _ow;
+    _srcrect.h = _oh;
+
+    _image = 0;
+    _filter = SMPEGfilter_null();
+    _filter_mutex = SDL_CreateMutex();
 }
 
 MPEGvideo:: ~MPEGvideo()
@@ -175,13 +214,13 @@ MPEGvideo:: ~MPEGvideo()
     /* Free actual video stream */
     if( _stream )
         DestroyVidStream( _stream );
-}
 
+    /* Free overlay */
+    if(_image) SDL_FreeYUVOverlay(_image);
 
-void
-MPEGvideo:: Loop(bool toggle)
-{
-    _stream->loopFlag = toggle;
+    /* Release filter */
+    SDL_DestroyMutex(_filter_mutex);
+    _filter->destroy(_filter);
 }
 
 /* Simple thread play function */
@@ -190,9 +229,17 @@ int Play_MPEGvideo( void *udata )
     MPEGvideo *mpeg = (MPEGvideo *)udata;
 
     /* Get the time the playback started */
-    mpeg->_stream->realTimeStart = ReadSysClock();
-    mpeg->play_time = 0.0;
+    mpeg->_stream->realTimeStart += ReadSysClock();
 
+#ifdef TIME_MPEG
+    int start_frames, stop_frames;
+    int total_frames;
+    Uint32 start_time, stop_time;
+    float total_time;
+
+    start_frames = mpeg->_stream->totNumFrames;
+    start_time = SDL_GetTicks();
+#endif
     while( mpeg->playing )
     {
         int mark = mpeg->_stream->totNumFrames;
@@ -205,15 +252,22 @@ int Play_MPEGvideo( void *udata )
 
         if( mpeg->_stream->film_has_ended )
         {
-            if( mpeg->_stream->loopFlag ) {
-                /* Rewind and start playback over */
-                mpeg->RewindStream();
-            } else {
-                mpeg->playing = false;
-            }
+            mpeg->playing = false;
         }
     }
-	return(0);
+    /* Get the time the playback stopped */
+    mpeg->_stream->realTimeStart -= ReadSysClock();
+#ifdef TIME_MPEG
+    stop_time = SDL_GetTicks();
+    stop_frames = mpeg->_stream->totNumFrames;
+    total_frames = (stop_frames-start_frames);
+    total_time = (float)(stop_time-start_time)/1000.0;
+    if ( total_time > 0 ) {
+        printf("%d frames in %2.2f seconds (%2.2f FPS)\n",
+               total_frames, total_time, (float)total_frames/total_time);
+    }
+#endif
+    return(0);
 }
 
 void
@@ -248,38 +302,71 @@ MPEGvideo:: Stop(void)
 }
 
 void
-MPEGvideo:: RewindStream(void)
+MPEGvideo:: Rewind(void)
 {
-    /* Reinitialize vid_stream pointers */
-    ResetVidStream( _stream );
-
-    mpeg->reset_stream();
-
+    Stop();
+    if ( _stream ) {
+      /* Reinitialize vid_stream pointers */
+      ResetVidStream( _stream );
 #ifdef ANALYSIS 
-    init_stats();
+      init_stats();
 #endif
-    /* Process start codes */
-    if( mpegVidRsrc( 0, _stream, 1 ) == NULL )
-    {
-        /* print something sensible here,
-           but we only get here if the file is changed while we
-           are decoding, right?
-        */
     }
 }
 
 void
-MPEGvideo:: Rewind(void)
+MPEGvideo:: ResetSynchro(double time)
 {
-    Stop();
-    RewindStream();
+  if( _stream )
+  {
+    _stream->_jumpFrame = -1;
+    _stream->realTimeStart = -time;
+    play_time = time;
+    if (time > 0) {
+	double oneframetime;
+	if (_stream->_oneFrameTime == 0)
+		oneframetime = 1.0 / _stream->_smpeg->_fps;	
+	else
+		oneframetime = _stream->_oneFrameTime;
+
+	/* time -> frame */
+	_stream->totNumFrames = (int)(time / oneframetime);
+
+	/* Set Current Frame To 0 & Frame Adjust Frag Set */
+	_stream->current_frame = 0;
+	_stream->need_frameadjust=true;
+    }
+  }
+}
+
+
+void
+MPEGvideo::Skip(float seconds)
+{
+  int frame;
+
+  /* Called only when there is no timestamp info in the MPEG */
+  /* This is quite slow however */
+  printf("Video: Skipping %f seconds...\n", seconds);  
+  frame = (int) (_fps * seconds);
+
+  if( _stream )
+  {
+    _stream->_jumpFrame = frame;
+    while( (_stream->totNumFrames < frame) &&
+	   ! _stream->film_has_ended )
+    {
+      mpegVidRsrc( 0, _stream, 0 );
+    }
+    ResetSynchro(0);
+  }
 }
 
 MPEGstatus
 MPEGvideo:: Status(void)
 {
-    if( _stream ) {
-        if( !_thread || (_stream->film_has_ended && !_stream->loopFlag) ) {
+    if ( _stream ) {
+        if( !_thread || (_stream->film_has_ended ) ) {
             return MPEG_STOPPED;
         } else {
             return MPEG_PLAYING;
@@ -292,10 +379,10 @@ bool
 MPEGvideo:: GetVideoInfo(MPEG_VideoInfo *info)
 {
     if ( info ) {
-        info->width = _w;
-        info->height = _h;
+        info->width = _ow;
+        info->height = _oh;
         if ( _stream ) {
-            info->current_frame = _stream->totNumFrames;
+            info->current_frame = _stream->current_frame;
 #ifdef CALCULATE_FPS
 
             /* Get the appropriate indices for the timestamps */
@@ -340,51 +427,38 @@ bool
 MPEGvideo:: SetDisplay(SDL_Surface *dst, SDL_mutex *lock,
                              MPEG_DisplayCallback callback)
 {
-    _surf = dst;
     _mutex = lock;
-    if( callback ) {
-        _callback = callback;
-    } else {
-        _callback = SDL_UpdateRect;
+    _dst = dst;
+    _callback = callback;
+    if ( _image ) {
+      SDL_FreeYUVOverlay(_image);
+    }
+    _image = SDL_CreateYUVOverlay(_srcrect.w, _srcrect.h, SDL_YV12_OVERLAY, dst);
+    if ( !_dstrect.w || !_dstrect.h ) {
+        _dstrect.w = dst->w;
+        _dstrect.h = dst->h;
     }
 
-    LUM_RANGE = 8;
-    CR_RANGE = 4;
-    CB_RANGE = 4;
+    if ( !_stream ) {
+        decodeInitTables();
 
-    lum_values = _lum;
-    cr_values  = _cr;
-    cb_values  = _cb;
+        InitCrop();
+        InitIDCT();
 
-    decodeInitTables();
+        _stream = NewVidStream( (unsigned int) BUF_LENGTH );
+        if( _stream ) {
+            _stream->_smpeg        = this;
+            _stream->ditherType    = FULL_COLOR_DITHER;
+            _stream->matched_depth = dst->format->BitsPerPixel;
 
-    InitColor();
-    {
-        /* Replace InitColorDisplay() */
-
-        InitColorDither(dst->format->BitsPerPixel,
-                        dst->format->Rmask,
-                        dst->format->Gmask,
-                        dst->format->Bmask);
-    }
-
-    InitCrop();
-
-    _stream = NewVidStream( (unsigned int) BUF_LENGTH );
-    //_stream = NewVidStream( /* 32K buffer size */ 32*1024 );
-    //_stream = NewVidStream( /* 16K buffer size */ 16*1024 );
-    if( _stream )
-    {
-        _stream->_smpeg        = this;
-        _stream->ditherType    = FULL_COLOR_DITHER;
-        _stream->matched_depth = dst->format->BitsPerPixel;
-
-        if( mpegVidRsrc( 0, _stream, 1 ) == NULL )
-        {
-            SetError("Not an MPEG video stream");
-            return false;
+            if( mpegVidRsrc( 0, _stream, 1 ) == NULL ) {
+                SetError("Not an MPEG video stream");
+                return false;
+            }
         }
     }
+    if ( ! InitPictImages(_stream, _w, _h, _dst) )
+        return false;
     return true;
 }
 
@@ -396,111 +470,126 @@ void
 MPEGvideo:: MoveDisplay( int x, int y )
 {
     SDL_mutexP( _mutex );
-    _x = x;
-    _y = y;
+    _dstrect.x = x;
+    _dstrect.y = y;
     SDL_mutexV( _mutex );
 }
 
 void
-MPEGvideo:: DoubleDisplay( bool toggle )
+MPEGvideo:: ScaleDisplayXY( int w, int h )
 {
-    _double = toggle;
+    SDL_mutexP( _mutex );
+    _dstrect.w = w;
+    _dstrect.h = h;
+    SDL_mutexV( _mutex );
 }
 
-
 void
-MPEGvideo:: RenderFrame( int frame, SDL_Surface* dst, int x, int y )
+MPEGvideo:: SetDisplayRegion(int x, int y, int w, int h)
 {
-    int saved_x, saved_y;
-    SDL_Surface *saved_surf;
+    SDL_mutexP( _mutex );
+    _srcrect.x = x;
+    _srcrect.y = y;
+    _srcrect.w = w;
+    _srcrect.h = h;
 
-    /* Save original mpeg stream parameters */
-    saved_x = _x;
-    saved_y = _y;
-    saved_surf = _surf;
+    if(_image)
+    {
+      SDL_FreeYUVOverlay(_image);
+      _image = SDL_CreateYUVOverlay(_srcrect.w, _srcrect.h, SDL_YV12_OVERLAY, _dst);
+    }
 
-    /* Set the parameters for this render */
-    _x = x;
-    _y = y;
-    _surf = dst;
+    SDL_mutexV( _mutex );
+}
 
-    if( _stream->totNumFrames > frame ) {
+/* API CHANGE: This function no longer takes a destination surface and x/y
+   You must use SetDisplay() and MoveDisplay() to set those attributes.
+*/
+void
+MPEGvideo:: RenderFrame( int frame )
+{
+    _stream->need_frameadjust = true;
+
+    if( _stream->current_frame > frame ) {
+        mpeg->rewind_stream();
+        mpeg->next_packet();
         Rewind();
     }
 
     _stream->_jumpFrame = frame;
 
-    while( (_stream->totNumFrames < frame) &&
+    while( (_stream->current_frame < frame) &&
            ! _stream->film_has_ended )
     {
         mpegVidRsrc( 0, _stream, 0 );
     }
 
     _stream->_jumpFrame = -1;
-
-    /* Now restore the parameters */
-    _x = saved_x;
-    _y = saved_y;
-    _surf = saved_surf;
 }
 
 void
 MPEGvideo:: RenderFinal(SDL_Surface *dst, int x, int y)
 {
+    SDL_Surface *saved_dst;
     int saved_x, saved_y;
-    SDL_Surface *saved_surf;
 
+    /* This operation can only be performed when stopped */
     Stop();
 
-    if( ! _stream->film_has_ended )
-    {
+    /* Set (and save) the destination and location */
+    saved_dst = _dst;
+    saved_x = _dstrect.x;
+    saved_y = _dstrect.y;
+    SetDisplay(dst, _mutex, _callback);
+    MoveDisplay(x, y);
+
+    if ( ! _stream->film_has_ended ) {
         /* Search for the last "group of pictures" start code */
         Uint32 start_code;
+        MPEGstream_marker * marker, * oldmarker;
 
-        mpeg->reset_stream();
+        marker = 0;
         start_code = mpeg->copy_byte();
         start_code <<= 8;
         start_code |= mpeg->copy_byte();
         start_code <<= 8;
         start_code |= mpeg->copy_byte();
+
         while ( ! mpeg->eof() ) {
             start_code <<= 8;
             start_code |= mpeg->copy_byte();
             if ( start_code == GOP_START_CODE ) {
-                mpeg->mark_data(-4);
+	          oldmarker = marker;
+        	  marker = mpeg->new_marker(-4);
+        	  if( oldmarker ) mpeg->delete_marker( oldmarker );
+       		  mpeg->garbage_collect();
             }
         }
 
         /* Set the stream to the last spot marked */
-        if ( ! mpeg->seek_marker() ) {
-            mpeg->reset_stream();
+        if ( ! mpeg->seek_marker( marker ) ) {
+            mpeg->rewind_stream();
+            mpeg->next_packet();
         }
+
+        mpeg->delete_marker( marker );
         _stream->buf_length = 0;
         _stream->bit_offset = 0;
 
         /* Process all frames without displaying any */
         _stream->_skipFrame = 1;
-        RenderFrame( INT_MAX, dst, x, y );
+
+        RenderFrame( INT_MAX );
+
+        mpeg->garbage_collect();
     }
 
-    /* Save original mpeg stream parameters */
-    saved_x = _x;
-    saved_y = _y;
-    saved_surf = _surf;
+    /* Display the frame */
+    DisplayFrame(_stream);
 
-    /* Set the parameters for this render */
-    _x = x;
-    _y = y;
-    _surf = dst;
-
-    /* Display the last frame processed */
-    DisplayCurrentFrame( _stream );
-
-    /* Now restore the parameters */
-    _x = saved_x;
-    _y = saved_y;
-    _surf = saved_surf;
+    /* Restore the destination and location */
+    SetDisplay(saved_dst, _mutex, _callback);
+    MoveDisplay(saved_x, saved_y);
 }
 
-
 /* EOF */
