@@ -48,15 +48,16 @@
  * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
  */
 
-
-
 #include <math.h>
 #include "video.h"
 #include "proto.h"
 #include "dither.h"
-#include <sys/time.h>
 #include "SDL_timer.h"
 
+#ifdef USE_ATI
+#include "vhar128.h"
+#endif
+
 #ifdef __STDC__
 #include <stdlib.h>
 #include <string.h>
@@ -79,16 +80,6 @@
    -lsh@cs.brown.edu (Loring Holden)
 */
 
-/* Range values for lum, cr, cb. */
-int LUM_RANGE;
-int CR_RANGE;
-int CB_RANGE;
-
-/* Arrays holding quantized value ranged for lum, cr, and cb. */
-int *lum_values;
-int *cr_values;
-int *cb_values;
-
 /* Frame Rate Info */
 extern int framerate;
 
@@ -96,150 +87,9 @@ extern int framerate;
 /* Cheat on Vid rates, round to 30, and use 30 if illegal value 
    Except for 9, where Xing means 15, and given their popularity, we'll
    be nice and do it */
-static int VidRateNum[16]={ 30, 24, 24, 25, 30, 30, 50, 60, 
+static double VidRateNum[16]={ 30, 23.97, 24, 25, 29.97, 30, 50, 59.94, 
                             60, 15, 30, 30, 30, 30, 30, 30 };
 
-/* Luminance and chrominance lookup tables */
-static double *L_tab, *Cr_r_tab, *Cr_g_tab, *Cb_g_tab, *Cb_b_tab;
-
-
-/*
- *--------------------------------------------------------------
- *
- * InitColor --
- *
- *	Initialize lum, cr, and cb quantized range value arrays.
- *      Also initializes the lookup tables for the possible
- *      values of lum, cr, and cb.
- *    Color values from ITU-R BT.470-2 System B, G and SMPTE 170M
- *    see InitColorDither in 16bits.c for more
- *
- * Results: 
- *      None.
- *
- * Side effects:
- *      None.
- *
- *--------------------------------------------------------------
- */
-
-void InitColor()
-{
-  int i, CR, CB;
-
-  if ( L_tab ) free(L_tab);
-  L_tab    = (double *)malloc(LUM_RANGE*sizeof(double)); 
-  if ( Cr_r_tab ) free(Cr_r_tab);
-  Cr_r_tab = (double *)malloc(CR_RANGE*sizeof(double));
-  if ( Cr_g_tab ) free(Cr_g_tab);
-  Cr_g_tab = (double *)malloc(CR_RANGE*sizeof(double));
-  if ( Cb_g_tab ) free(Cb_g_tab);
-  Cb_g_tab = (double *)malloc(CB_RANGE*sizeof(double));
-  if ( Cb_b_tab ) free(Cb_b_tab);
-  Cb_b_tab = (double *)malloc(CB_RANGE*sizeof(double));
-
-  if (L_tab == NULL    || Cr_r_tab == NULL ||
-      Cr_g_tab == NULL || Cb_g_tab == NULL ||
-      Cb_b_tab == NULL) {
-    fprintf(stderr, "Could not alloc memory in InitColor\n");
-    exit(1);
-  }
-
-  for (i=0; i<LUM_RANGE; i++) {
-    lum_values[i]  = ((i * 256) / (LUM_RANGE)) + (256/(LUM_RANGE*2));
-    L_tab[i] = lum_values[i];
-    if (gammaCorrectFlag) {
-      L_tab[i] = GAMMA_CORRECTION(L_tab[i]);
-    }
-  }
-  
-  for (i=0; i<CR_RANGE; i++) {
-    register double tmp;
-    if (chromaCorrectFlag) {
-      tmp = ((i * 256) / (CR_RANGE)) + (256/(CR_RANGE*2));
-      Cr_r_tab[i] = (int) (0.419/0.299) * CHROMA_CORRECTION128D(tmp - 128.0);
-      Cr_g_tab[i] = (int) -(0.299/0.419) * CHROMA_CORRECTION128D(tmp - 128.0);
-      cr_values[i] = CHROMA_CORRECTION256(tmp);
-    } else {
-      tmp = ((i * 256) / (CR_RANGE)) + (256/(CR_RANGE*2));
-      Cr_r_tab[i] = (int)  (0.419/0.299) * (tmp - 128.0);
-      Cr_g_tab[i] = (int) -(0.299/0.419) * (tmp - 128.0);
-      cr_values[i] = (int) tmp;
-    }
-  }
-
-  for (i=0; i<CB_RANGE; i++) {
-    register double tmp;
-    if (chromaCorrectFlag) {
-      tmp = ((i * 256) / (CB_RANGE)) + (256/(CB_RANGE*2));
-      Cb_g_tab[i] = (int) -(0.114/0.331) * CHROMA_CORRECTION128D(tmp - 128.0);
-      Cb_b_tab[i] = (int)  (0.587/0.331) * CHROMA_CORRECTION128D(tmp - 128.0);
-      cb_values[i] = CHROMA_CORRECTION256(tmp);
-    } else {
-      tmp = ((i * 256) / (CB_RANGE)) + (256/(CB_RANGE*2));
-      Cb_g_tab[i] = (int) -(0.114/0.331) * (tmp - 128.0);
-      Cb_b_tab[i] = (int)  (0.587/0.331) * (tmp - 128.0);
-      cb_values[i] = (int) tmp;
-    }
-  }
-
-}
-
-
-/*
- *--------------------------------------------------------------
- *
- * ConvertColor --
- *
- *	Given a l, cr, cb tuple, converts it to r,g,b.
- *
- * Results:
- *	r,g,b values returned in pointers passed as parameters.
- *
- * Side effects:
- *      None.
- *
- *--------------------------------------------------------------
- */
-
-void ConvertColor( unsigned int l, unsigned int cr, unsigned int cb,
-                   unsigned char* r, unsigned char* g, unsigned char* b )
-{
-    double fl, fcr, fcb, fr, fg, fb;
-
-    /*
-     * Old method w/o lookup table
-     *
-     * fl = 1.164*(((double) l)-16.0);
-     * fcr =  ((double) cr) - 128.0;
-     * fcb =  ((double) cb) - 128.0;
-     *
-     * fr = fl + (1.366 * fcr);
-     * fg = fl - (0.700 * fcr) - (0.334 * fcb);
-     * fb = fl + (1.732 * fcb);
-     */
-
-    fl = L_tab[l];
-
-    fr = fl + Cr_r_tab[cr];
-    fg = fl + Cr_g_tab[cr] + Cb_g_tab[cb];
-    fb = fl + Cb_b_tab[cb];
-
-    if (fr < 0.0) fr = 0.0;
-    else if (fr > 255.0) fr = 255.0;
-
-    if (fg < 0.0) fg = 0.0;
-    else if (fg > 255.0) fg = 255.0;
-
-    if (fb < 0.0) fb = 0.0;
-    else if (fb > 255.0) fb = 255.0;
-
-    *r = (unsigned char) fr;
-    *g = (unsigned char) fg;
-    *b = (unsigned char) fb;
-}
-
-
 #ifdef CALCULATE_FPS
 static inline void TimestampFPS( VidStream* vid_stream )
 {
@@ -294,12 +144,13 @@ inline double CurrentTime( VidStream* vid_stream )
     return now;
 }
 
-int timeSync( VidStream* vid_stream )
+int MPEGvideo::timeSync( VidStream* vid_stream )
 {
-    MPEGvideo* mpeg = (MPEGvideo*) vid_stream->_smpeg;
+    static double correction = -1;
 
     /* Update the number of frames displayed */
     vid_stream->totNumFrames++;
+    vid_stream->current_frame++;
 
     /* Do we need to initialize framerate? */
     if ( vid_stream->rate_deal < 0 ) {
@@ -322,7 +173,28 @@ int timeSync( VidStream* vid_stream )
     }
 
     /* Update the current play time */
-    mpeg->play_time += vid_stream->_oneFrameTime;
+    play_time += vid_stream->_oneFrameTime;
+
+    /* Synchronize using system timestamps */
+    if(vid_stream->current && vid_stream->current->show_time > 0){
+#ifdef DEBUG_TIMESTAMP_SYNC
+      fprintf(stderr, "video: time:%.3f  shift:%.3f\r",
+	      play_time,
+	      play_time - vid_stream->current->show_time);
+#endif
+      if(correction == -1)
+#ifdef STRANGE_SYNC_TEST
+       /* this forces us to maintain the offset we have at the begining
+          all the time, and is only usefull for testing */
+        correction = play_time - vid_stream->current->show_time;
+#else
+       correction = 0;
+#endif
+#ifdef USE_TIMESTAMP_SYNC
+      play_time = vid_stream->current->show_time + correction ;
+#endif
+      vid_stream->current->show_time = -1;
+    }
 
     /* If we are looking for a particular frame... */
     if( vid_stream->_jumpFrame > -1 )
@@ -347,10 +219,10 @@ int timeSync( VidStream* vid_stream )
         double time_behind;
 
         /* Calculate the frame time relative to real time */
-        time_behind = CurrentTime(vid_stream) - mpeg->Time();
+        time_behind = CurrentTime(vid_stream) - Time();
 
 #ifdef DEBUG_MPEG_SCHEDULING
-//printf("Frame %d: frame time: %f, real time: %f, time behind: %f\n", vid_stream->totNumFrames, mpeg->Time(), CurrentTime(vid_stream), time_behind);
+printf("Frame %d: frame time: %f, real time: %f, time behind: %f\n", vid_stream->totNumFrames, Time(), CurrentTime(vid_stream), time_behind);
 #endif
 
         /* Allow up to MAX_FUDGE_TIME of delay in output */
@@ -386,7 +258,7 @@ printf("A little behind, skipping %d frames\n", vid_stream->_skipFrame);
 printf("Way too far behind, losing time sync...\n");
 #endif
 #if 0 // This results in smoother video, but sync's terribly on slow machines
-                mpeg->play_time = CurrentTime(vid_stream) - (MAX_FUDGE_TIME*2);
+                play_time = CurrentTime(vid_stream) - (MAX_FUDGE_TIME*2);
 #endif
             }
 #ifdef SLOW_START_SCHEDULING
@@ -408,87 +280,95 @@ printf("A lot behind, skipping %d frames\n", vid_stream->_skipFrame);
 
 /* Do the hard work of copying from the video stream working buffer to the
    screen display and then calling the update callback.
-   This has been optimized for writing to 16-bit SDL surfaces.
 */
-void DisplayCurrentFrame( VidStream* vid_stream )
+void MPEGvideo::DisplayFrame( VidStream * vid_stream )
 {
-    unsigned char* l = vid_stream->current->luminance;
-    unsigned char* Cr = vid_stream->current->Cr;
-    unsigned char* Cb = vid_stream->current->Cb;
-    unsigned char* disp;
-    MPEGvideo* mpeg = (MPEGvideo*) vid_stream->_smpeg;
+  SMPEG_FilterInfo info;
 
-    if ( SDL_MUSTLOCK(mpeg->_surf) ) {
-        if ( SDL_LockSurface(mpeg->_surf) < 0 ) {
-            return;
-        }
-    }
-    disp = (unsigned char*) mpeg->_surf->pixels;
-    disp += mpeg->_x * 2;
-    disp += mpeg->_surf->pitch * mpeg->_y;
+  if ( _filter_mutex )
+    SDL_mutexP( _filter_mutex );
 
-    if ( mpeg->_mutex )
-        SDL_mutexP( mpeg->_mutex );
+  /* Get a pointer to _image pixels */
+  if ( SDL_LockYUVOverlay( _image ) ) {
+    return;
+  }
 
-    if( mpeg->_double )
-    {
-#ifdef USE_INTERLACED_VIDEO
-        static int start = 1;
-        if ( mpeg->_surf->format->BytesPerPixel == 2 ) {
-        	Twox2Color16DitherImageModInterlace( l, Cr, Cb, disp,
-                  vid_stream->v_size, vid_stream->h_size,
-                 (mpeg->_surf->pitch / 2) - (vid_stream->h_size * 2), start );
-	}
-        start = !start;
+  /* Compute additionnal info for the filter */
+  if((_filter->flags & SMPEG_FILTER_INFO_PIXEL_ERROR) && vid_stream->current->mb_qscale)
+  {
+    register int x, y;
+    register Uint16 * ptr;
+
+    /* Compute quantization error for each pixel */
+    info.yuv_pixel_square_error = (Uint16 *) malloc(_w*_h*12/8*sizeof(Uint16));
+
+    ptr =  info.yuv_pixel_square_error;
+    for(y = 0; y < _h; y++)
+      for(x = 0; x < _w; x++)
+	*ptr++ = (Uint16) (((Uint32) vid_stream->noise_base_matrix[x & 7][y & 7] * 
+			    vid_stream->current->mb_qscale[((y>>4) * (_w>>4)) + (x >> 4)]) >> 8);
+  }
+  
+  if((_filter->flags & SMPEG_FILTER_INFO_MB_ERROR) && vid_stream->current->mb_qscale)
+  {
+    /* Retreive macroblock quantization error info */
+    info.yuv_mb_square_error = vid_stream->current->mb_qscale;
+  }
+    
+  if( _filter )
+  {
+    SDL_Overlay src;
+    Uint16 pitches[3];
+    Uint8 *pixels[3];
+
+    /* Fill in an SDL YV12 overlay structure for the source */
+#ifdef USE_ATI
+    vhar128_lockimage(vid_stream->ati_handle, vid_stream->current->image, &src);
 #else
-        if ( mpeg->_surf->format->BytesPerPixel == 2 ) {
-            Twox2Color16DitherImageMod( l, Cr, Cb, disp,
-                  vid_stream->v_size, vid_stream->h_size,
-                 (mpeg->_surf->pitch / 2) - (vid_stream->h_size * 2) );
-        }
-        if ( mpeg->_surf->format->BytesPerPixel == 4 ) {
-            Twox2Color32DitherImageMod( l, Cr, Cb, disp,
-                  vid_stream->v_size, vid_stream->h_size,
-                 (mpeg->_surf->pitch / 4) - (vid_stream->h_size * 2) );
-        }
+    src.format = SDL_YV12_OVERLAY;
+    src.w = _w;
+    src.h = _h;
+    src.planes = 3;
+    pitches[0] = _w;
+    pitches[1] = _w / 2;
+    pitches[2] = _w / 2;
+    src.pitches = pitches;
+    pixels[0] = vid_stream->current->image;
+    pixels[1] = vid_stream->current->image + pitches[0] * _h;
+    pixels[2] = vid_stream->current->image + pitches[0] * _h +
+                                             pitches[1] * _h / 2;
+    src.pixels = pixels;
 #endif
-        if ( SDL_MUSTLOCK(mpeg->_surf) ) {
-            SDL_UnlockSurface(mpeg->_surf);
-        }
-        mpeg->_callback( mpeg->_surf, mpeg->_x, mpeg->_y,
-                     vid_stream->h_size * 2, vid_stream->v_size * 2 );
-    }
-    else
-    {
-#ifdef USE_INTERLACED_VIDEO
-        static int start = 1;
-        if ( mpeg->_surf->format->BytesPerPixel == 2 ) {
-        	Color16DitherImageModInterlace( l, Cr, Cb, disp,
-                         vid_stream->v_size, vid_stream->h_size,
-                         (mpeg->_surf->pitch / 2) - vid_stream->h_size, start );
-	}
-        start = !start;
-#else
-        if ( mpeg->_surf->format->BytesPerPixel == 2 ) {
-            Color16DitherImageMod( l, Cr, Cb, disp,
-                         vid_stream->v_size, vid_stream->h_size,
-                         (mpeg->_surf->pitch / 2) - vid_stream->h_size );
-        } else
-        if ( mpeg->_surf->format->BytesPerPixel == 4 ) {
-            Color32DitherImageMod( l, Cr, Cb, disp,
-                         vid_stream->v_size, vid_stream->h_size,
-                         (mpeg->_surf->pitch / 4) - vid_stream->h_size );
-        }
-#endif
-        if ( SDL_MUSTLOCK(mpeg->_surf) ) {
-            SDL_UnlockSurface(mpeg->_surf);
-        }
-        mpeg->_callback( mpeg->_surf, mpeg->_x, mpeg->_y,
-                         vid_stream->h_size, vid_stream->v_size );
-    }
 
-    if ( mpeg->_mutex )
-        SDL_mutexV( mpeg->_mutex );
+    _filter->callback(_image, &src, &_srcrect, &info, _filter->data );
+
+#ifdef USE_ATI
+    vhar128_unlockimage(vid_stream->ati_handle, vid_stream->current->image, &src);
+#endif
+  }
+
+  /* Now display the image */
+  if ( _mutex )
+    SDL_mutexP( _mutex );
+
+  SDL_DisplayYUVOverlay(_image, &_dstrect);
+
+  if ( _callback )
+    _callback(_dst, _dstrect.x, _dstrect.y, _dstrect.w, _dstrect.h);
+
+  SDL_UnlockYUVOverlay( _image );
+
+  if( _filter )
+  {
+    if( _filter->flags & SMPEG_FILTER_INFO_PIXEL_ERROR )
+      free(info.yuv_pixel_square_error);
+  }
+
+  if ( _filter_mutex )
+    SDL_mutexV( _filter_mutex );
+  
+  if ( _mutex )
+    SDL_mutexV( _mutex );
 }
 
 /*
@@ -508,17 +388,33 @@ void DisplayCurrentFrame( VidStream* vid_stream )
  */
 
 
-void ExecuteDisplay( VidStream* vid_stream )
+void MPEGvideo::ExecuteDisplay( VidStream* vid_stream )
 {
     if( ! vid_stream->_skipFrame )
     {
-        DisplayCurrentFrame(vid_stream);
+      DisplayFrame(vid_stream);
+
 #ifdef CALCULATE_FPS
-        TimestampFPS(vid_stream);
+      TimestampFPS(vid_stream);
 #endif
     }
     timeSync( vid_stream );
 }
 
 
+SMPEG_Filter *
+MPEGvideo:: Filter(SMPEG_Filter * filter)
+{
+  SMPEG_Filter * old_filter;
+
+  old_filter = _filter;
+  if ( _filter_mutex )
+    SDL_mutexP( _filter_mutex );
+  _filter = filter;
+  if ( _filter_mutex )
+    SDL_mutexV( _filter_mutex );
+
+  return(old_filter);
+}
+
 /* EOF */
