@@ -83,8 +83,7 @@ typedef Uint8 UINT8;
 
 /* Set ring buffer size. */
 
-//#define RING_BUF_SIZE 5
-#define RING_BUF_SIZE 3
+#define RING_BUF_SIZE 5
 
 /* Macros for picture code type. */
 
@@ -146,21 +145,21 @@ typedef Uint8 UINT8;
 #define IS_2x2_DITHER(a) ((a) == Twox2_DITHER || (a) == FULL_COLOR2_DITHER || (a) == GRAY2_DITHER || (a) == (GRAY2562_DITHER))
 #endif
 
-/* External declaration of row,col to zig zag conversion matrix. */
-
 /* Brown - changed to const int because it is a help variable */
 extern const int scan[][8];
 
-/* Temporary definition of time stamp structure. */
-
-typedef int TimeStamp;
-
 /* Structure with reconstructed pixel values. */
 
 typedef struct pict_image {
+#ifdef USE_ATI
+  struct vhar128_image *image;
+#else
+  unsigned char *image;                  /* YV12 format image  */
   unsigned char *luminance;              /* Luminance plane.   */
   unsigned char *Cr;                     /* Cr plane.          */
   unsigned char *Cb;                     /* Cb plane.          */
+#endif
+  unsigned short int *mb_qscale;         /* macroblock info    */
   int locked;                            /* Lock flag.         */
   TimeStamp show_time;                   /* Presentation time. */
 } PictImage;
@@ -234,7 +233,11 @@ typedef struct macroblock {
 /* Block structure. */
 
 typedef struct block {
+#ifdef USE_ATI
+  long int dct_recon[6][130];            /* Reconstructed dct runs & levels */
+#else
   short int dct_recon[8][8];             /* Reconstructed dct coeff matrix. */
+#endif
   short int dct_dc_y_past;               /* Past lum. dc dct coefficient.   */
   short int dct_dc_cr_past;              /* Past cr dc dct coefficient.     */
   short int dct_dc_cb_past;              /* Past cb dc dct coefficient.     */
@@ -270,6 +273,10 @@ typedef struct vid_stream {
   int buf_length;                              /* Length of remaining buffer.*/
   unsigned int *buf_start;                     /* Pointer to buffer start.   */
 
+/* VC - beginning of added variables for noise computation */
+  short noise_base_matrix[8][8];               /* Square quantization error  */
+/* VC - end of added variables */
+
 /* Brown - beginning of added variables that used to be static or global */
   int max_buf_length;                          /* Max length of buffer.      */
   int film_has_ended;                          /* Boolean - film has ended   */
@@ -293,8 +300,7 @@ typedef struct vid_stream {
   PictImage *ring[RING_BUF_SIZE];              /* Ring buffer of frames.     */
 
 /* KR - beginning of added variables */
-  int loopFlag;
-  int rate_deal;
+  double rate_deal;
   int _skipFrame;
   double _skipCount;
   int _jumpFrame;
@@ -310,6 +316,17 @@ typedef struct vid_stream {
   int timestamp_index;
 #endif
 /* SL - end of added variables */
+/* begining of added variables for system stream based sync */
+  double timestamp;
+  unsigned int *timestamp_mark;
+  bool timestamp_used;
+/* begining of added variables */
+  bool need_frameadjust;
+  int  current_frame;
+
+#ifdef USE_ATI
+  unsigned int ati_handle;
+#endif
 
 } VidStream;   
 
@@ -407,7 +424,11 @@ extern unsigned int cacheMiss[8][8];
 #endif
 
 #if !defined(LITTLE_ENDIAN_ARCHITECTURE) && !defined(BIG_ENDIAN_ARCHITECTURE)
-#include </usr/include/endian.h>
+#ifdef WIN32
+#undef  BIG_ENDIAN_ARCHITECTURE
+#define LITTLE_ENDIAN_ARCHITECTURE
+#else
+#include <endian.h>
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 #undef  BIG_ENDIAN_ARCHITECTURE
 #define LITTLE_ENDIAN_ARCHITECTURE 1
@@ -417,6 +438,7 @@ extern unsigned int cacheMiss[8][8];
 #define BIG_ENDIAN_ARCHITECTURE 1
 #endif
 #endif
+#endif
 
 #if !defined(LITTLE_ENDIAN_ARCHITECTURE) && !defined(BIG_ENDIAN_ARCHITECTURE)
 #error Unknown endianism of architecture
@@ -425,4 +447,9 @@ extern unsigned int cacheMiss[8][8];
 #ifdef __alpha__
 #define SIXTYFOUR_BIT
 #endif
+
+/* Warnings that may help in debugging MPEG streams */
+//#define VERBOSE_WARNINGS
+//#define VERBOSE_DEBUG
+
 #endif /* video.h already included */
